__author__ = 'mnowotka'

from optparse import make_option
import traceback
import os, re

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import NoArgsCommand
from django.core.management.color import no_style
from django.core.management.sql import emit_post_sync_signal
from django.db import connections, router, transaction, models, DEFAULT_DB_ALIAS
from django.utils.datastructures import SortedDict
from django.utils.importlib import import_module

def custom_sql_for_model(model, style, connection):
    opts = model._meta
    app_dir = os.path.normpath(os.path.join(os.path.dirname(models.get_app(model._meta.app_label).__file__), 'sql'))
    output = []

    # Post-creation SQL should come before any initial SQL data is loaded.
    # However, this should not be done for models that are unmanaged or
    # for fields that are part of a parent model (via model inheritance).
    if opts.managed:
        post_sql_fields = [f for f in opts.local_fields if hasattr(f, 'post_create_sql')]
        for f in post_sql_fields:
            output.extend(f.post_create_sql(style, model._meta.db_table))

    # Some backends can't execute more than one SQL statement at a time,
    # so split into separate statements.
    statements = re.compile(r";[ \t]*$", re.M)

    # Find custom SQL, if it's available.
    backend_name = connection.settings_dict['ENGINE'].split('.')[-1]
    sql_files = [os.path.join(app_dir, "%s.%s.sql" % (opts.object_name.lower(), backend_name)),
                 os.path.join(app_dir, "%s.sql" % opts.object_name.lower())]
    for sql_file in sql_files:
        if os.path.exists(sql_file):
            fp = open(sql_file, 'U')
            for statement in statements.split(fp.read().decode(settings.FILE_CHARSET)):
                # Remove any comments from the file
                statement = re.sub(ur"--.*([\n\Z]|$)", "", statement)
                if statement.strip():
                    output.append(statement + u";")
            fp.close()

    return output

class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('--noinput', action='store_false', dest='interactive', default=True,
            help='Tells Django to NOT prompt the user for input of any kind.'),
        make_option('--no-initial-data', action='store_false', dest='load_initial_data', default=True,
            help='Tells Django not to load any initial data after database synchronization.'),
        make_option('--database', action='store', dest='database',
            default=DEFAULT_DB_ALIAS, help='Nominates a database to synchronize. '
                'Defaults to the "default" database.'),
    )
    help = "Create the database tables for all apps in INSTALLED_APPS whose tables haven't already been created."

    def handle_noargs(self, **options):

        verbosity = int(options.get('verbosity'))
        interactive = options.get('interactive')
        show_traceback = options.get('traceback')
        load_initial_data = options.get('load_initial_data')

        self.style = no_style()

        # Import the 'management' module within each installed app, to register
        # dispatcher events.
        for app_name in settings.INSTALLED_APPS:
            try:
                import_module('.management', app_name)
            except ImportError as exc:
                # This is slightly hackish. We want to ignore ImportErrors
                # if the "management" module itself is missing -- but we don't
                # want to ignore the exception if the management module exists
                # but raises an ImportError for some reason. The only way we
                # can do this is to check the text of the exception. Note that
                # we're a bit broad in how we check the text, because different
                # Python implementations may not use the same text.
                # CPython uses the text "No module named management"
                # PyPy uses "No module named myproject.myapp.management"
                msg = exc.args[0]
                if not msg.startswith('No module named') or 'management' not in msg:
                    raise

        db = options.get('database')
        connection = connections[db]
        cursor = connection.cursor()

        # Get a list of already installed *models* so that references work right.
        tables = connection.introspection.table_names()
        seen_models = connection.introspection.installed_models(tables)
        created_models = set()
        pending_references = {}

        # Build the manifest of apps and models that are to be synchronized
        all_models = [
            (app.__name__.split('.')[-2],
                [m for m in models.get_models(app, include_auto_created=True)
                if router.allow_syncdb(db, m)])
            for app in models.get_apps()
        ]

        def model_installed(model):
            opts = model._meta
            converter = connection.introspection.table_name_converter
            return not ((converter(opts.db_table) in tables) or
                (opts.auto_created and converter(opts.auto_created._meta.db_table) in tables))

        manifest = SortedDict(
            (app_name, list(filter(model_installed, model_list)))
            for app_name, model_list in all_models
        )

        # Create the tables for each model
        if verbosity >= 1:
            self.stdout.write("Creating tables ...\n")
        for app_name, model_list in manifest.items():
            for model in model_list:
                # Create the model's database table, if it doesn't already exist.
                if verbosity >= 3:
                    self.stdout.write("Processing %s.%s model\n" % (app_name, model._meta.object_name))
                sql, references = connection.creation.sql_create_model(model, self.style, seen_models)
                seen_models.add(model)
                created_models.add(model)
                for refto, refs in references.items():
                    pending_references.setdefault(refto, []).extend(refs)
                    if refto in seen_models:
                        sql.extend(connection.creation.sql_for_pending_references(refto, self.style, pending_references))
                sql.extend(connection.creation.sql_for_pending_references(model, self.style, pending_references))
                if verbosity >= 1 and sql:
                    self.stdout.write("Creating table %s\n" % model._meta.db_table)
                for statement in sql:
                    cursor.execute(statement)
                tables.append(connection.introspection.table_name_converter(model._meta.db_table))

        transaction.commit_unless_managed(using=db)

        # Send the post_syncdb signal, so individual apps can do whatever they need
        # to do at this point.
        emit_post_sync_signal(created_models, verbosity, interactive, db)

        # The connection may have been closed by a syncdb handler.
        cursor = connection.cursor()

        # Install custom SQL for the app (but only if this
        # is a model we've just created)
        if verbosity >= 1:
            self.stdout.write("Installing custom SQL ...\n")
        for app_name, model_list in manifest.items():
            for model in model_list:
                if model in created_models:
                    custom_sql = custom_sql_for_model(model, self.style, connection)
                    if custom_sql:
                        if verbosity >= 2:
                            self.stdout.write("Installing custom SQL for %s.%s model\n" % (app_name, model._meta.object_name))
                        try:
                            for sql in custom_sql:
                                cursor.execute(sql)
                        except Exception as e:
                            self.stderr.write("Failed to install custom SQL for %s.%s model: %s\n" % \
                                                (app_name, model._meta.object_name, e))
                            if show_traceback:
                                traceback.print_exc()
                            transaction.rollback_unless_managed(using=db)
                        else:
                            transaction.commit_unless_managed(using=db)
                    else:
                        if verbosity >= 3:
                            self.stdout.write("No custom SQL for %s.%s model\n" % (app_name, model._meta.object_name))

        if verbosity >= 1:
            self.stdout.write("Installing indexes ...\n")
        # Install SQL indices for all newly created models
        for app_name, model_list in manifest.items():
            for model in model_list:
                if model in created_models:
                    index_sql = connection.creation.sql_indexes_for_model(model, self.style)
                    if index_sql:
                        if verbosity >= 2:
                            self.stdout.write("Installing index for %s.%s model\n" % (app_name, model._meta.object_name))
                        try:
                            for sql in index_sql:
                                cursor.execute(sql)
                        except Exception as e:
                            self.stderr.write("Failed to install index for %s.%s model: %s\n" % \
                                                (app_name, model._meta.object_name, e))
                            transaction.rollback_unless_managed(using=db)
                        else:
                            transaction.commit_unless_managed(using=db)

        # Load initial_data fixtures (unless that has been disabled)
        if load_initial_data:
            call_command('loaddata', 'initial_data', verbosity=verbosity,
                         database=db, skip_validation=True)



__author__ = 'mnowotka'

from django.core.management.base import BaseCommand
from optparse import make_option
from clint.textui import progress
from rdkit import Chem
from rdkit import DataStructs
from rdkit.Chem.AtomPairs import Pairs
from rdkit.Chem import MACCSkeys
from rdkit.Chem import rdMolDescriptors
from rdkit import rdBase

#------------------------------------------------------------------------------

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--in', dest='in_file',
            help='Input file'),
        make_option('--out', dest='out_file',
            default='out.fps.gz', help='Output file'),
        make_option('--fp_type', dest='fp_type',
            default='morgan', help='Type of fingerprint'),
        make_option('--n_bits', dest='n_bits',
            default=2048, help='Fingerprint length in bits'),
        make_option('--radius', dest='radius',
            default=2, help='Radius for the Morgan algorithm'),
        )
    help = ("Convert SDF to fps")
    args = '[--in, --out, --n_bits]'

#------------------------------------------------------------------------------

    def getFileHandle(self, path, mode):
        import os
        fileName, fileExtension = os.path.splitext(path)
        if fileExtension == '.gz':
            import gzip
            f = gzip.open(path, mode)
        elif fileExtension == '.bz2':
            import bz2
            f = bz2.BZ2File(path, mode)
        else:
            f = open(path, mode)
        return f

#------------------------------------------------------------------------------

    def handle(self, *args, **options):

        in_file = options.get('in_file')
        out_file = options.get('out_file')
        n_bits = options.get('n_bits')
        verbosity = options.get('verbosity')
        radius = options.get('radius')
        fp_type = options.get('fp_type')

        if verbosity > 1:
            print self.help

        inp = self.getFileHandle(in_file, 'rb')
        out = self.getFileHandle(out_file, 'wb')

        suppl = Chem.ForwardSDMolSupplier(inp)
        out.write("#FPS1\n#num_bits=%s\n#software=RDKit/%s\n" % (n_bits, rdBase.rdkitVersion))

        for i, mol in progress.mill(enumerate(suppl), expected_size=5):
            if mol:
                idx = i
                if mol.HasProp('chembl_id'):
                    idx = mol.GetProp('chembl_id')
                else:
                    try:
                        idx = Chem.InchiToInchiKey(Chem.MolToInchi(mol))
                    except:
                        pass
                if fp_type == 'morgan':
                    fp = rdMolDescriptors.GetMorganFingerprintAsBitVect(mol,radius,nBits=n_bits)
                elif fp_type == 'pair':
                    fp = Pairs.GetAtomPairFingerprintAsBitVect(mol)
                elif fp_type == 'maccs':
                    fp = MACCSkeys.GenMACCSKeys(mol)
                out.write("%s\t%s\n" % (DataStructs.BitVectToFPSText(fp), idx))

        if verbosity > 1:
            print "Conversion completed."

#------------------------------------------------------------------------------


__author__ = 'mnowotka'

import os
from django.core.management.base import BaseCommand
from optparse import make_option
from django.db import DEFAULT_DB_ALIAS
from django.conf import settings
from django import db
from django.db import connections
from django.utils.datastructures import SortedDict
from django.core.management.commands.dumpdata import sort_dependencies

#-----------------------------------------------------------------------------------------------------------------------

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--database', dest='sourceDatabase', default=DEFAULT_DB_ALIAS, help='Source database'),
        make_option('--dumpfile', dest='dumpfile', default=None, help='Location of dump file.'),
        make_option('--datalimit', dest='dataLimit', default=10000, help='Data limit'),
        make_option('--app', dest='app', default='chembl_migration_model', help='App to be exported'),
        )
    help = ("Prepare configuration file for ora2pg tool.")
    args = '[appname appname.ModelName ...]'

    confTemplate = '''
ORACLE_HOME     %s
ORACLE_DSN      dbi:Oracle:host=%s;sid=%s;port=%s
ORACLE_USER     %s
ORACLE_PWD      %s
SCHEMA          %s
TABLES          %s
USER_GRANTS     1
DEBUG           0
EXPORT_SCHEMA   0
COMPILE_SCHEMA  0
TYPE            DATA
DATA_LIMIT      %s
CASE_SENSITIVE  0
OUTPUT          %s
DATA_TYPE	    DATE:date,LONG:text,LONG RAW:bytea,CLOB:text,NCLOB:text,BLOB:bytea,BFILE:bytea,RAW:bytea,ROWID:oid,FLOAT:double precision,DEC:decimal,DECIMAL:decimal,DOUBLE PRECISION:double precision,INT:integer,INTEGER:integer,REAL:real,SMALLINT:smallint,BINARY_FLOAT:double precision,BINARY_DOUBLE:double precision,TIMESTAMP:timestamp,TIMESTAMP WITH TIME ZONE:timestamp with time zone,TIMESTAMP WITH LOCAL TIME ZONE:timestamp
BZIP2   /bin/bzip2
GEN_USER_PWD    0
FKEY_DEFERRABLE 0
DEFER_FKEY      0
DROP_FKEY       0
DROP_INDEXES    0
PG_NUMERIC_TYPE 0
DEFAULT_NUMERIC NUMERIC
KEEP_PKEY_NAMES 1
DISABLE_TABLE_TRIGGERS 1
NOESCAPE        0
DISABLE_SEQUENCE        0
ORA_SENSITIVE   0
PLSQL_PGSQL     1
ORA_RESERVED_WORDS      audit,comment
FILE_PER_CONSTRAINT     0
FILE_PER_INDEX          0
FILE_PER_TABLE  0
TRANSACTION     serializable
PG_SUPPORTS_WHEN                1
PG_SUPPORTS_INSTEADOF   0
FILE_PER_FUNCTION       0
TRUNCATE_TABLE  0
FORCE_OWNER     0
STANDARD_CONFORMING_STRINGS     0
THREAD_COUNT            0
ALLOW_CODE_BREAK        1
XML_PRETTY      1
FDW_SERVER      orcl
ENABLE_MICROSECOND      0
DISABLE_COMMENT         1
    '''

#-----------------------------------------------------------------------------------------------------------------------

    def handle(self, *args, **options):
        from django.db.models import get_app

        #TODO : Check export mode
        db.reset_queries()
        sourceDatabase = options.get('sourceDatabase')
        dataLimit = options.get('dataLimit')
        app = get_app(options.get('app'))

        con = connections[sourceDatabase]
        if con.vendor != 'oracle':
            print "Source database has to be oracle."
            return

        user = settings.DATABASES[sourceDatabase]['USER']
        passwd = settings.DATABASES[sourceDatabase]['PASSWORD']
        host = settings.DATABASES[sourceDatabase]['HOST']
        port = settings.DATABASES[sourceDatabase]['PORT']
        name = settings.DATABASES[sourceDatabase]['NAME']

        app_list = SortedDict((app, None) for app in [app])
        tables = []
        sorted = sort_dependencies(app_list.items())
        lastObjectName = sorted[-1].__name__
        filename = lastObjectName + ".postgresql_psycopg2.sql"
        chemblSQLPath  = os.path.join(os.path.dirname(app.__file__),'sql', filename)
        location = chemblSQLPath
        oracleHome = os.environ['ORACLE_HOME']

        if options.get('dumpfile'):
            if not options.get('dumpfile').endswith('.sql'):
                location = os.path.join(options.get('dumpfile'), filename)
            else:
                location = options.get('dumpfile')


        for model in reversed(sorted):
            if not model._meta.managed:
                continue
            tables.append(model._meta.db_table)

        print self.confTemplate % (oracleHome, host, name, port, user, passwd, user, " ".join(tables), dataLimit, location)

        if location != chemblSQLPath:
            print "different! location = " + location + ", chemblSQLPath = " + chemblSQLPath
            f = open(location, 'w')
            f.close()
            os.symlink(location, chemblSQLPath)


#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

import gzip
from chembl_business_model.models import *


try:
    import cPickle as pickle
except ImportError:
    import pickle

#-----------------------------------------------------------------------------------------------------------------------

class Ontology:
    def containsTerm(self, term):
        pass

    def getIdsForTerm(self, term):
        pass

    def getAllTerms(self):
        pass

    def addTerm(self, term, id):
        pass

    def getOntology(self):
        pass

    def merge(self, ontology):
        pass

    def removeTerm(self, term):
        pass

    def fetch(self):
        pass

    def load(self, path):
        pass

    def save(self, path):
        pass

#-----------------------------------------------------------------------------------------------------------------------

class DictionaryBasedOntology(Ontology):
    def __init__(self):
        self.dict = {}

    def containsTerm(self, term):
        return term in self.dict

    def getIdsForTerm(self, term):
        return self.dict[term]

    def getAllTerms(self):
        return self.dict.keys()

    def addTerm(self, term, id):
        if term in dict:
            if isinstance(id, list):
                self.dict[term] += id
            else:
                self.dict[term].append(id)
        else:
            if isinstance(id, list):
                self.dict[term] = id
            else:
                self.dict[term] = [id]

    def getOntology(self):
        return self.dict.items()

    def merge(self, ontology):
        self.dict = dict(self.dict.items() + ontology.getOntology())

    def removeTerm(self, term):
        if term in self.dict:
            del self.dict[term]

    def save(self, path):
        f = gzip.open(path, 'wb')
        pickle.dump(self.dict, f)
        f.close()

    def load(self, path):
        f = gzip.open('file.txt.gz', 'rb')
        self.dict = pickle.load(f)
        f.close()

#-----------------------------------------------------------------------------------------------------------------------

class MoleculeOntology(Ontology):
    def fetch(self):
        for k, v in MoleculeDictionary.objects.filter(pref_name__isnull=False).values_list('pref_name', 'molregno'):
            self.addTerm(k, v)

#-----------------------------------------------------------------------------------------------------------------------

class MoleculeSynonymsOntology(Ontology):
    def fetch(self):
        for k, v in MoleculeSynonyms.objects.filter(synonyms__isnull=False).values_list('synonyms', 'molsyn_id'):
            self.addTerm(k, v)

#-----------------------------------------------------------------------------------------------------------------------

class TargetOntology(Ontology):
    def fetch(self):
        for k, v in TargetDictionary.objects.filter(pref_name__isnull=False).values_list('pref_name', 'tid'):
            self.addTerm(k, v)

#-----------------------------------------------------------------------------------------------------------------------

class AssayOntology(Ontology):
    def fetch(self):
        for k, v in Assays.objects.filter(description__isnull=False).values_list('description', 'assay_id'):
            self.addTerm(k, v)

#-----------------------------------------------------------------------------------------------------------------------

class ComponentSynonymsOntology(Ontology):
    def fetch(self):
        for k, v in ComponentSynonyms.objects.filter(component_synonym__isnull=False).values_list('component_synonym', 'compsyn_id'):
            self.addTerm(k, v)

#-----------------------------------------------------------------------------------------------------------------------


__author__ = 'mnowotka'

from chembl_core_model.models import CompoundMols
from chembl_core_model.models import CompoundStructures
from django.core.management.base import BaseCommand
from optparse import make_option
from clint.textui import progress
import gzip
from django import db
try:
    import cPickle as pickle
except ImportError:
    import pickle

#------------------------------------------------------------------------------

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--eps', default=0.3, dest='eps',
            help='Specifies the output serialization format for migration.'),
        make_option('--min_points', dest='min_points',
            default=100, help='Target database'),
        make_option('--out', dest='out_file',
            default='clusters.pkl.gz', help='Output file'),
        make_option('--in', dest='input',
            default=None, help='Input file'),
        )
    help = ("Cluster compounds in database.")
    args = '[--eps, --min_points, --out, --in]'

#------------------------------------------------------------------------------

    molecules = None
    infile = None
    filename = None
    UNCLASSIFIED = False
    NOISE = None

#------------------------------------------------------------------------------

    def _region_query(self, mol_id, eps):
        n_retries = 0
        while True:
            try:
                smiles = CompoundStructures.objects.only(
                                'canonical_smiles').get(pk=1).canonical_smiles

                if not smiles:
                    return []
                return [sim[0] for sim in
                        CompoundMols.objects.all().similar_to(
                            smiles, int((1.0 - eps) * 100)).values_list('pk')]
            except Exception as e:
                n_retries += 1
                db.close_connection()
                if n_retries >= 3:
                    self.save()
                    raise e
                pass

#------------------------------------------------------------------------------

    def _expand_cluster(self, pk, cluster_id, eps, min_points):
        seeds = self._region_query(pk, eps)
        if len(seeds) < min_points:
            self.molecules[pk] = self.NOISE
            return False
        else:
            self.molecules[pk] = cluster_id
            for seed in seeds:
                self.molecules[seed] = cluster_id

            while len(seeds):
                current_point = seeds[0]
                results = self._region_query(current_point, eps)
                n_results = len(results)
                if n_results >= min_points:
                    for result in results:
                        clas = self.molecules[result]
                        if clas == self.UNCLASSIFIED or clas == self.NOISE:
                            if clas == self.UNCLASSIFIED:
                                seeds.append(result)
                            self.molecules[result] = cluster_id
                seeds = seeds[1:]
            return True

#------------------------------------------------------------------------------

    def dbscan(self, eps, min_points):
        print "DBSCAN started..."
        cluster_id = 1
        for pk, cl in progress.bar(self.molecules.iteritems(),
                    label="clustering... ", expected_size=len(self.molecules)):
            if self.molecules[pk] == self.UNCLASSIFIED:
                if self._expand_cluster(pk, cluster_id, eps, min_points):
                    cluster_id += 1
        print "DBSCAN finished!"

#------------------------------------------------------------------------------

    def save(self):
        f = gzip.open(self.filename, 'wb')
        pickle.dump(self.molecules, f)
        f.close()

#------------------------------------------------------------------------------

    def load(self):
        f = gzip.open(self.filename, 'rb')
        self.molecules = pickle.load(f)
        f.close()

#------------------------------------------------------------------------------

    def handle(self, *args, **options):
        eps = options.get('eps')
        min_points = options.get('min_points')
        self.filename = options.get('out_file')
        self.infile = options.get('input')

        if not self.infile:
            print "prefetching data..."

            self.molecules = dict([(mol[0], self.UNCLASSIFIED) for mol in
                                    CompoundMols.objects.all().values_list('pk')])

            self.save()
            print "done"
        else:
            self.load()

        try:
            self.dbscan(eps, min_points)
        finally:
            self.save()

#------------------------------------------------------------------------------


#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'mnowotka'

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup

setup(
    name='chembl_extras',
    version='0.6.0',
    author='Michal Nowotka',
    author_email='mnowotka@ebi.ac.uk',
    description='Python package providing extra collection of django custom management commands for use with ChEMBL and some classes for use in future',
    url='https://www.ebi.ac.uk/chembldb/index.php/ws',
    license='Apache Software License',
    packages=['chembl_extras',
              'chembl_extras.management',
              'chembl_extras.management.commands'],
    long_description=open('README.rst').read(),
    install_requires=[
        'chembl_business_model',
        'biopython',
        'clint>=0.3.3',
    ],
    include_package_data=False,
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Scientific/Engineering :: Chemistry'],
    zip_safe=False,
)


__author__ = 'mnowotka'

import gzip
from Bio import trie
from Bio import triefind
from chembl_business_model import utils
from chembl_business_model.models import CompoundStructures

class MoleculeFinder():

#-----------------------------------------------------------------------------------------------------------------------

    def __init__(self):
        triefind.DEFAULT_BOUNDARY_CHARS = triefind.DEFAULT_BOUNDARY_CHARS.translate(None, utils.INCHI_SPECIAL_CHARS)

        self.index = trie.trie()

        for inchi in CompoundStructures.objects.all().values_list('standard_inchi', 'molecule_id'):
            for chunk in inchi[0].split('/')[1:]:
                if len(chunk) > 2:
                    if self.index.get(str(chunk)):
                        self.index[str(chunk)].append(inchi[1])
                    else:
                        self.index[str(chunk)] = [inchi[1]]

#-----------------------------------------------------------------------------------------------------------------------

    def find(self, string):
        if len(string) < 3:
            return []

        if 'InChI=1S'.find(string) != (-1) and len(string) > 3:
            return CompoundStructures.objects.all()

        if string[1:-1].find('/') == (-1):
            if string.find('*') == (-1):
                if string.endswith('/'):
                    return self.index.get(str(string.translate(None, '/')))
                else:
                    res = triefind.find(string, self.index)
                    lengths = map(lambda x: x[2] - x[1], res)
                    if not lengths:
                        return []
                    maxlen = max(lengths)
                    indices = [i for i, x in enumerate(lengths) if x == maxlen]
                    return sum(map(lambda x: self.index[str(x[0])], [i for j, i in enumerate(res) if j in indices]), [])
            else:
                return  sum(
                    map(lambda x: x[1], self.index.get_approximate(string.translate(None, '/'), string.count('*'))), [])

        else:
            res = triefind.find(string, self.index)
            keys = map(lambda x: x[0], res)
            score = dict()
            for key in keys:
                for mol in self.index.get(key):
                    if not score.get(mol):
                        score[mol] = 1
                    else:
                        score[mol] += 1

            maxscore = max(score.values())
            return map(lambda x: x[0], [x for x in score.iteritems() if x[1] == maxscore])


#-----------------------------------------------------------------------------------------------------------------------

mf = MoleculeFinder()


__author__ = 'mnowotka'

from chembl_core_model.models import CompoundStructures
from django.core.management.base import BaseCommand
from optparse import make_option
from clint.textui import progress
from django.conf import settings
from django.db import DEFAULT_DB_ALIAS
import gzip

#------------------------------------------------------------------------------

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--chebi_id', default=True, dest='chebi_id',
            help='Include chebi_id.'),
        make_option('--chembl_id', default=True, dest='chembl_id',
            help='Include chembl_id.'),
        make_option('--downgraded', default=True, dest='downgraded',
            help='Include downgraded compounds.'),
        make_option('--out', dest='out_file',
            default='chembl.sdf.gz', help='Output file'),
        make_option('--database', dest='db',
            default=DEFAULT_DB_ALIAS, help='database')
        )
    help = ("Export all molecules from ChEMBL to one compressed *.sdf file.")
    args = '[--chebi_id, --chembl_id, --downgraded, --out]'

#------------------------------------------------------------------------------

    def handle(self, *args, **options):

        if settings.DEBUG:
            print "Django is in debug mode, which causes memory leak. Set settings.DEBUG to False and run again."
            return

        filename = options.get('out_file')
        chebi_id = options.get('chebi_id')
        chembl_id = options.get('chembl_id')
        downgraded = options.get('downgraded')
        db = options.get('db')
        verbosity = options.get('verbosity')

        if verbosity > 1:
            print self.help

        filters = {'molecule__chembl__entity_type':'COMPOUND'}

        if not downgraded:
            filters['molecule__downgraded'] = False

        fields = ['molfile']
        mol_template = '%s\n'

        if chembl_id:
            fields.append('molecule__chembl__chembl_id')
            mol_template += '> <chembl_id>\n%s\n\n'

        if chebi_id:
            fields.append('molecule__chebi_id')
            mol_template += '> <chebi_id>\n%s\n\n'

        mol_template += '$$$$\n'

        structures = CompoundStructures.objects.using(db).filter(**filters)
        n_structures = structures.count()

        if verbosity > 1:
            print "Found %s structures to export" % n_structures

        if not n_structures:
            print "Nothing to export..."

        else:
            f = gzip.open(filename, 'wb')
            try:
                for raw in progress.bar(structures.values_list(*fields).iterator(),
                            label="exporting... ", expected_size=n_structures):
                    f.write(mol_template % raw)

            finally:
                f.close()

        if verbosity > 1:
            print "Exporting done."

#------------------------------------------------------------------------------